#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''  
   Copyright (c) 2019 Yahoo Japan Corporation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
'''

import sys
import wikitextparser as wtp
import xml.etree.ElementTree as ET
from flashtext import KeywordProcessor
import argparse

ap = argparse.ArgumentParser()
ap.add_argument('-c', '--causesectionfile', type=str, required=True)
ap.add_argument('-e', '--effectsectionfile', type=str, required=True)
ap.add_argument('-o', '--ceoutfile', type=str, required=True)
ap.add_argument('-s', '--sep', type=str, default="\t")
ap.add_argument('--pagestart', type=str, default="<page>")
ap.add_argument('--pageend', type=str, default="</page>")
ap.add_argument('--titlepat', type=str, default=".//title")
ap.add_argument('--textpat', type=str, default=".//text")
ap.add_argument('--newline', type=str, default=" ||| ")
ap.add_argument('--case_sensitive', type=bool, default=False)
ap.add_argument('--causeseclabel', type=str, default="c")
ap.add_argument('--effectseclabel', type=str, default="e")
args = ap.parse_args()

def prepare_keyword_processor():
    keyword_processor = KeywordProcessor(case_sensitive=args.case_sensitive)
    with open(args.causesectionfile, 'r') as f:
        for l in f:
            sec = l.strip()
            keyword_processor.add_keyword(sec, args.causeseclabel)
    with open(args.effectsectionfile, 'r') as f:
        for l in f:
            sec = l.strip()
            keyword_processor.add_keyword(sec, args.effectseclabel)
    return keyword_processor

def process(xml, kp):
    root = ET.fromstring(''.join(xml))
    title = root.find(args.titlepat).text.strip()
    text = root.find(args.textpat).text
    if not text:
        return False
    parsed = None
    try:
        parsed = wtp.parse(text)
    except:
        print("Parsing failed for: %s" % title, file=sys.stderr)
        import traceback
        traceback.print_exc()
        return False
    for sec in parsed.sections:
        sectitle = sec.title.strip()
        seclabels = set(kp.extract_keywords(sectitle))
        if len(seclabels) == 1:
            body = args.newline.join(sec.string.splitlines())
            body = body.replace("\t", " ")
            print("%s\t%s\t%d\t%s\t%s" % (title, sectitle, sec.level, seclabels.pop(), body))
            return title
    return False

def main():
    kp = prepare_keyword_processor()
    xml = [] 
    record_flag = False
    ces = set()
    for l in sys.stdin:
        if record_flag:
            xml.append(l)
        if args.pagestart in l:
            record_flag = True
            xml.append(l)
        elif args.pageend in l:
            record_flag = False
            title = process(xml, kp)
            if title:
                ces.add(title)
            xml = []
    with open(args.ceoutfile, 'w') as f:
        print("\n".join(ces), file=f)

if __name__ == "__main__":
    main()

